@@ -8,8 +8,7 @@
 This module adds the functionality of storing and retrieving :class:`~basyx.aas.model.base.Identifiable` objects
 in a CouchDB.
 
-The :class:`~.CouchDBBackend` takes care of updating and committing objects from and to the CouchDB, while the
-:class:`~CouchDBObjectStore` handles adding, deleting and otherwise managing the AAS objects in a specific CouchDB.
+The :class:`~CouchDBObjectStore` handles adding, deleting and otherwise managing the AAS objects in a specific CouchDB.
 """
 import threading
 import weakref
@@ -21,7 +20,6 @@ import logging
 import json
 import urllib3  # type: ignore
 
-from . import backends
 from ..adapter.json import json_serialization, json_deserialization
 from basyx.aas import model
 
@@ -30,146 +28,6 @@ logger = logging.getLogger(__name__)
 _http_pool_manager = urllib3.PoolManager()
 
 
-class CouchDBBackend(backends.Backend):
-    """
-    This Backend stores each Identifiable object as a single JSON document in the configured CouchDB database. Each
-    document's id is build from the object's identifier. The document's contents comprise a single property ``data``,
-    containing the JSON serialization of the BaSyx Python SDK object. The :ref:`adapter.json <adapter.json.__init__>`
-    package is used for serialization and deserialization of objects.
-    """
-    @classmethod
-    def update_object(cls,
-                      updated_object: model.Referable,
-                      store_object: model.Referable,
-                      relative_path: List[str]) -> None:
-
-        if not isinstance(store_object, model.Identifiable):
-            raise CouchDBSourceError("The given store_object is not Identifiable, therefore cannot be found "
-                                     "in the CouchDB")
-        url = CouchDBBackend._parse_source(store_object.source)
-
-        try:
-            data = CouchDBBackend.do_request(url)
-        except CouchDBServerError as e:
-            if e.code == 404:
-                raise KeyError("No Identifiable found in CouchDB at {}".format(url)) from e
-            raise
-
-        updated_store_object = data['data']
-        set_couchdb_revision(url, data["_rev"])
-        store_object.update_from(updated_store_object)
-
-    @classmethod
-    def commit_object(cls,
-                      committed_object: model.Referable,
-                      store_object: model.Referable,
-                      relative_path: List[str]) -> None:
-        if not isinstance(store_object, model.Identifiable):
-            raise CouchDBSourceError("The given store_object is not Identifiable, therefore cannot be found "
-                                     "in the CouchDB")
-        url = CouchDBBackend._parse_source(store_object.source)
-        # We need to get the revision of the object, if it already exists, otherwise we cannot write to the Couchdb
-        if get_couchdb_revision(url) is None:
-            raise CouchDBConflictError("No revision found for the given object. Try calling `update` on it.")
-
-        data = json.dumps({'data': store_object, "_rev": get_couchdb_revision(url)},
-                          cls=json_serialization.AASToJsonEncoder)
-
-        try:
-            response = CouchDBBackend.do_request(
-                url, method='PUT', additional_headers={'Content-type': 'application/json'}, body=data.encode('utf-8'))
-            set_couchdb_revision(url, response["rev"])
-        except CouchDBServerError as e:
-            if e.code == 409:
-                raise CouchDBConflictError("Could not commit changes to id {} due to a concurrent modification in the "
-                                           "database.".format(store_object.id)) from e
-            elif e.code == 404:
-                raise KeyError("Object with id {} was not found in the CouchDB at {}"
-                               .format(store_object.id, url)) from e
-            raise
-
-    @classmethod
-    def _parse_source(cls, source: str) -> str:
-        """
-        Parses the source parameter of a model.Referable object
-
-        :param source: Source string of the model.Referable object
-        :return: URL to the document
-        :raises CouchDBBackendSourceError, if the source has the wrong format
-        """
-        if source.startswith("couchdbs://"):
-            url = source.replace("couchdbs://", "https://", 1)
-        elif source.startswith("couchdb://"):
-            url = source.replace("couchdb://", "http://", 1)
-        else:
-            raise CouchDBSourceError("Source has wrong format. "
-                                     "Expected to start with {couchdb://, couchdbs://}, got {" + source + "}")
-        return url
-
-    @classmethod
-    def do_request(cls, url: str, method: str = "GET", additional_headers: Optional[Dict[str, str]] = None,
-                   body: Optional[bytes] = None) -> MutableMapping[str, Any]:
-        """
-        Perform an HTTP(S) request to the CouchDBServer, parse the result and handle errors
-
-        :param url: The HTTP or HTTPS URL to request
-        :param method: The HTTP method for the request
-        :param additional_headers: Additional headers to insert into the request. The default headers include
-            'connection: keep-alive', 'accept-encoding: ...', 'authorization: basic ...', 'Accept: ...'.
-        :param body: Request body for POST, PUT, and PATCH requests
-        :return: The parsed JSON data if the request ``method`` is other than 'HEAD' or the response headers for 'HEAD'
-            requests
-        """
-        url_parts = urllib.parse.urlparse(url)
-        host = url_parts.scheme + url_parts.netloc
-        auth = _credentials_store.get(host)
-        headers = urllib3.make_headers(keep_alive=True, accept_encoding=True,
-                                       basic_auth="{}:{}".format(*auth) if auth else None)
-        headers['Accept'] = 'application/json'
-        headers.update(additional_headers if additional_headers is not None else {})
-        try:
-            response = _http_pool_manager.request(method, url, headers=headers, body=body)
-        except (urllib3.exceptions.TimeoutError, urllib3.exceptions.SSLError, urllib3.exceptions.ProtocolError) as e:
-            raise CouchDBConnectionError("Error while connecting to the CouchDB server: {}".format(e)) from e
-        except urllib3.exceptions.HTTPError as e:
-            raise CouchDBResponseError("Error while connecting to the CouchDB server: {}".format(e)) from e
-
-        if not (200 <= response.status < 300):
-            logger.debug("Request %s %s finished with HTTP status code %s.",
-                         method, url, response.status)
-            if response.headers.get('Content-type', None) != 'application/json':
-                raise CouchDBResponseError("Unexpected Content-type header {} of response from CouchDB server"
-                                           .format(response.headers.get('Content-type', None)))
-
-            if method == 'HEAD':
-                raise CouchDBServerError(response.status, "", "", "HTTP {}".format(response.status))
-
-            try:
-                data = json.loads(response.data.decode('utf-8'))
-            except json.JSONDecodeError:
-                raise CouchDBResponseError("Could not parse error message of HTTP {}"
-                                           .format(response.status))
-            raise CouchDBServerError(response.status, data['error'], data['reason'],
-                                     "HTTP {}: {} (reason: {})".format(response.status, data['error'], data['reason']))
-
-        # Check response & parse data
-        logger.debug("Request %s %s finished successfully.", method, url)
-        if method == 'HEAD':
-            return response.headers
-
-        if response.headers.get('Content-type') != 'application/json':
-            raise CouchDBResponseError("Unexpected Content-type header")
-        try:
-            data = json.loads(response.data.decode('utf-8'), cls=json_deserialization.AASFromJsonDecoder)
-        except json.JSONDecodeError as e:
-            raise CouchDBResponseError("Could not parse CouchDB server response as JSON data.") from e
-        return data
-
-
-backends.register_backend("couchdb", CouchDBBackend)
-backends.register_backend("couchdbs", CouchDBBackend)
-
-
 # Global registry for credentials for CouchDB Servers
 _credentials_store: Dict[str, Tuple[str, str]] = {}
 # Note: The HTTPPasswordMgr is not thread safe during writing, should be thread safe for reading only.
@@ -182,8 +40,7 @@ def register_credentials(url: str, username: str, password: str):
     .. Warning::
 
         Do not use this function, while other threads may be accessing the credentials via the
-        :class:`~.CouchDBObjectStore` or update or commit functions of :class:`~.basyx.aas.model.base.Referable`
-        objects!
+        :class:`~.CouchDBObjectStore`!
 
     :param url: Toplevel URL
     :param username: Username to that CouchDB instance
@@ -268,7 +125,7 @@ class CouchDBObjectStore(model.AbstractObjectStore):
         """
 
         try:
-            CouchDBBackend.do_request("{}/{}".format(self.url, self.database_name), 'HEAD')
+            self._do_request("{}/{}".format(self.url, self.database_name), 'HEAD')
         except CouchDBServerError as e:
             # If an HTTPError is raised, re-raise it, unless it is a 404 error and we are requested to create the
             # database
@@ -280,7 +137,7 @@ class CouchDBObjectStore(model.AbstractObjectStore):
 
         # Create database
         logger.info("Creating CouchDB database %s/%s ...", self.url, self.database_name)
-        CouchDBBackend.do_request("{}/{}".format(self.url, self.database_name), 'PUT')
+        self._do_request("{}/{}".format(self.url, self.database_name), 'PUT')
 
     def get_identifiable_by_couchdb_id(self, couchdb_id: str) -> model.Identifiable:
         """
@@ -293,7 +150,7 @@ class CouchDBObjectStore(model.AbstractObjectStore):
         # Create and issue HTTP request (raises HTTPError on status != 200)
 
         try:
-            data = CouchDBBackend.do_request(
+            data = self._do_request(
                 "{}/{}/{}".format(self.url, self.database_name, urllib.parse.quote(couchdb_id, safe='')))
         except CouchDBServerError as e:
             if e.code == 404:
@@ -305,7 +162,6 @@ class CouchDBObjectStore(model.AbstractObjectStore):
         if not isinstance(obj, model.Identifiable):
             raise CouchDBResponseError("The CouchDB document with id {} does not contain an identifiable AAS object."
                                        .format(couchdb_id))
-        self.generate_source(obj)  # Generate the source parameter of this object
         set_couchdb_revision("{}/{}/{}".format(self.url, self.database_name, urllib.parse.quote(couchdb_id, safe='')),
                              data["_rev"])
 
@@ -314,12 +170,8 @@ class CouchDBObjectStore(model.AbstractObjectStore):
         with self._object_cache_lock:
             if obj.id in self._object_cache:
                 old_obj = self._object_cache[obj.id]
-                # If the source does not match the correct source for this CouchDB backend, the object seems to belong
-                # to another backend now, so we return a fresh copy
-                if old_obj.source == obj.source:
-                    old_obj.update_from(obj)
-                    return old_obj
-
+                old_obj.update_from(obj)
+                return old_obj
         self._object_cache[obj.id] = obj
         return obj
 
@@ -351,7 +203,7 @@ class CouchDBObjectStore(model.AbstractObjectStore):
         # Create and issue HTTP request (raises HTTPError on status != 200)
 
         try:
-            response = CouchDBBackend.do_request(
+            response = self._do_request(
                 "{}/{}/{}".format(self.url, self.database_name, self._transform_id(x.id)),
                 'PUT',
                 {'Content-type': 'application/json'},
@@ -364,7 +216,6 @@ class CouchDBObjectStore(model.AbstractObjectStore):
             raise
         with self._object_cache_lock:
             self._object_cache[x.id] = x
-        self.generate_source(x)  # Set the source of the object
 
     def discard(self, x: model.Identifiable, safe_delete=False) -> None:
         """
@@ -394,7 +245,7 @@ class CouchDBObjectStore(model.AbstractObjectStore):
             # ETag response header
             try:
                 logger.debug("fetching the current object revision for deletion ...")
-                headers = CouchDBBackend.do_request(
+                headers = self._do_request(
                     "{}/{}/{}".format(self.url, self.database_name, self._transform_id(x.id)), 'HEAD')
                 rev = headers['ETag'][1:-1]
             except CouchDBServerError as e:
@@ -403,7 +254,7 @@ class CouchDBObjectStore(model.AbstractObjectStore):
                         from e
                 raise
         try:
-            CouchDBBackend.do_request(
+            self._do_request(
                 "{}/{}/{}?rev={}".format(self.url, self.database_name, self._transform_id(x.id), rev),
                 'DELETE')
         except CouchDBServerError as e:
@@ -419,7 +270,65 @@ class CouchDBObjectStore(model.AbstractObjectStore):
                                                   self._transform_id(x.id)))
         with self._object_cache_lock:
             del self._object_cache[x.id]
-        x.source = ""
+
+    @classmethod
+    def _do_request(cls, url: str, method: str = "GET", additional_headers: Dict[str, str] = {},
+                    body: Optional[bytes] = None) -> MutableMapping[str, Any]:
+        """
+        Perform an HTTP(S) request to the CouchDBServer, parse the result and handle errors
+
+        :param url: The HTTP or HTTPS URL to request
+        :param method: The HTTP method for the request
+        :param additional_headers: Additional headers to insert into the request. The default headers include
+            'connection: keep-alive', 'accept-encoding: ...', 'authorization: basic ...', 'Accept: ...'.
+        :param body: Request body for POST, PUT, and PATCH requests
+        :return: The parsed JSON data if the request ``method`` is other than 'HEAD' or the response headers for 'HEAD'
+            requests
+        """
+        url_parts = urllib.parse.urlparse(url)
+        host = url_parts.scheme + url_parts.netloc
+        auth = _credentials_store.get(host)
+        headers = urllib3.make_headers(keep_alive=True, accept_encoding=True,
+                                       basic_auth="{}:{}".format(*auth) if auth else None)
+        headers['Accept'] = 'application/json'
+        headers.update(additional_headers)
+        try:
+            response = _http_pool_manager.request(method, url, headers=headers, body=body)
+        except (urllib3.exceptions.TimeoutError, urllib3.exceptions.SSLError, urllib3.exceptions.ProtocolError) as e:
+            raise CouchDBConnectionError("Error while connecting to the CouchDB server: {}".format(e)) from e
+        except urllib3.exceptions.HTTPError as e:
+            raise CouchDBResponseError("Error while connecting to the CouchDB server: {}".format(e)) from e
+
+        if not (200 <= response.status < 300):
+            logger.debug("Request %s %s finished with HTTP status code %s.",
+                         method, url, response.status)
+            if response.headers.get('Content-type', None) != 'application/json':
+                raise CouchDBResponseError("Unexpected Content-type header {} of response from CouchDB server"
+                                           .format(response.headers.get('Content-type', None)))
+
+            if method == 'HEAD':
+                raise CouchDBServerError(response.status, "", "", "HTTP {}".format(response.status))
+
+            try:
+                data = json.loads(response.data.decode('utf-8'))
+            except json.JSONDecodeError:
+                raise CouchDBResponseError("Could not parse error message of HTTP {}"
+                                           .format(response.status))
+            raise CouchDBServerError(response.status, data['error'], data['reason'],
+                                     "HTTP {}: {} (reason: {})".format(response.status, data['error'], data['reason']))
+
+        # Check response & parse data
+        logger.debug("Request %s %s finished successfully.", method, url)
+        if method == 'HEAD':
+            return response.headers
+
+        if response.headers.get('Content-type') != 'application/json':
+            raise CouchDBResponseError("Unexpected Content-type header")
+        try:
+            data = json.loads(response.data.decode('utf-8'), cls=json_deserialization.AASFromJsonDecoder)
+        except json.JSONDecodeError as e:
+            raise CouchDBResponseError("Could not parse CouchDB server response as JSON data.") from e
+        return data
 
     def __contains__(self, x: object) -> bool:
         """
@@ -441,7 +350,7 @@ class CouchDBObjectStore(model.AbstractObjectStore):
         logger.debug("Checking existence of object with id %s in database ...", repr(x))
 
         try:
-            CouchDBBackend.do_request(
+            self._do_request(
                 "{}/{}/{}".format(self.url, self.database_name, self._transform_id(identifier)), 'HEAD')
         except CouchDBServerError as e:
             if e.code == 404:
@@ -458,7 +367,7 @@ class CouchDBObjectStore(model.AbstractObjectStore):
                               (see ``_do_request()`` for details)
         """
         logger.debug("Fetching number of documents from database ...")
-        data = CouchDBBackend.do_request("{}/{}".format(self.url, self.database_name))
+        data = self._do_request("{}/{}".format(self.url, self.database_name))
         return data['doc_count']
 
     def __iter__(self) -> Iterator[model.Identifiable]:
@@ -483,7 +392,7 @@ class CouchDBObjectStore(model.AbstractObjectStore):
 
         # Fetch a list of all ids and construct Iterator object
         logger.debug("Creating iterator over objects in database ...")
-        data = CouchDBBackend.do_request("{}/{}/_all_docs".format(self.url, self.database_name))
+        data = self._do_request("{}/{}/_all_docs".format(self.url, self.database_name))
         return CouchDBIdentifiableIterator(self, (row['id'] for row in data['rows']))
 
     @staticmethod
@@ -497,17 +406,6 @@ class CouchDBObjectStore(model.AbstractObjectStore):
             identifier = urllib.parse.quote(identifier, safe='')
         return identifier
 
-    def generate_source(self, identifiable: model.Identifiable):
-        """
-        Generates the source string for an :class:`~basyx.aas.model.base.Identifiable` object that is backed
-        by the Couchdb
-
-        :param identifiable: Identifiable object
-        """
-        source: str = self.url.replace("https://", "couchdbs://").replace("http://", "couchdb://")
-        source += "/" + self.database_name + "/" + self._transform_id(identifiable.id)
-        identifiable.source = source
-
 
 # #################################################################################################
 # Custom Exception classes for reporting errors during interaction with the CouchDB server
@@ -516,11 +414,6 @@ class CouchDBError(Exception):
     pass
 
 
-class CouchDBSourceError(CouchDBError):
-    """Exception raised when the source has the wrong format"""
-    pass
-
-
 class CouchDBConnectionError(CouchDBError):
     """Exception raised when the CouchDB server could not be reached"""
     pass
