@@ -11,7 +11,6 @@ from typing import Callable, Dict, Iterable, List, Optional, Type, TypeVar
 from collections import OrderedDict
 
 from basyx.aas import model
-from basyx.aas.backend import backends
 from basyx.aas.model import Identifier, Identifiable
 from basyx.aas.examples.data import example_aas
 
@@ -42,7 +41,7 @@ class KeyTest(unittest.TestCase):
         self.assertEqual(model.Key(model.KeyTypes.MULTI_LANGUAGE_PROPERTY, "0"), model.Key.from_referable(mlp2))
         with self.assertRaises(ValueError) as cm:
             model.Key.from_referable(mlp1)
-        self.assertEqual("Can't create Key for MultiLanguageProperty without an id_short!", str(cm.exception))
+        self.assertEqual("Can't create Key value for MultiLanguageProperty without an id_short!", str(cm.exception))
         mlp1.id_short = "mlp1"
         self.assertEqual(model.Key(model.KeyTypes.MULTI_LANGUAGE_PROPERTY, "mlp1"), model.Key.from_referable(mlp1))
 
@@ -52,28 +51,11 @@ class ExampleReferable(model.Referable):
         super().__init__()
 
 
-class ExampleRefereableWithNamespace(model.Referable, model.UniqueIdShortNamespace):
+class ExampleReferableWithNamespace(model.Referable, model.UniqueIdShortNamespace):
     def __init__(self):
         super().__init__()
 
 
-class MockBackend(backends.Backend):
-    @classmethod
-    def update_object(cls,
-                      updated_object: "Referable",  # type: ignore
-                      store_object: "Referable",  # type: ignore
-                      relative_path: List[str]) -> None: ...
-
-    @classmethod
-    def commit_object(cls,
-                      committed_object: "Referable",  # type: ignore
-                      store_object: "Referable",  # type: ignore
-                      relative_path: List[str]) -> None: ...
-
-    update_object = mock.Mock()
-    commit_object = mock.Mock()
-
-
 class ExampleIdentifiable(model.Identifiable):
     def __init__(self):
         super().__init__()
@@ -84,7 +66,6 @@ def generate_example_referable_tree() -> model.Referable:
     Generates an example referable tree, built like this:
 
         example_grandparent -> example_parent -> example_referable -> example_child -> example_grandchild
-        example_grandparent and example_grandchild both have an nonempty source, pointing to the mock-backend
 
     :return: example_referable
     """
@@ -98,7 +79,7 @@ def generate_example_referable_tree() -> model.Referable:
         :param child: Child to be added to the namespace sets of the Referable
         :return: The generated Referable
         """
-        referable = ExampleRefereableWithNamespace()
+        referable = ExampleReferableWithNamespace()
         referable.id_short = id_short
         if child:
             namespace_set = model.NamespaceSet(parent=referable, attribute_names=[("id_short", True)],
@@ -111,9 +92,6 @@ def generate_example_referable_tree() -> model.Referable:
     example_parent = generate_example_referable_with_namespace("exampleParent", example_referable)
     example_grandparent = generate_example_referable_with_namespace("exampleGrandparent", example_parent)
 
-    example_grandchild.source = "mockScheme:exampleGrandchild"
-    example_grandparent.source = "mockScheme:exampleGrandparent"
-
     return example_referable
 
 
@@ -157,97 +135,95 @@ class ReferableTest(unittest.TestCase):
         self.assertEqual('Referable must have an identifiable as root object and only parents that are referable',
                          str(cm.exception))
 
-    def test_update(self):
-        backends.register_backend("mockScheme", MockBackend)
-        example_referable = generate_example_referable_tree()
-        example_grandparent = example_referable.parent.parent
-        example_grandchild = example_referable.get_referable("exampleChild").get_referable("exampleGrandchild")
+    def test_get_identifiable_root(self):
+        ref_with_no_parent = ExampleReferableWithNamespace()
+        ref_with_no_parent.id_short = "NotNone"
 
-        # Test update with parameter "recursive=False"
-        example_referable.update(recursive=False)
-        MockBackend.update_object.assert_called_once_with(
-            updated_object=example_referable,
-            store_object=example_grandparent,
-            relative_path=["exampleGrandparent", "exampleParent", "exampleReferable"]
-        )
-        MockBackend.update_object.reset_mock()
+        identifiable = ExampleIdentifiable()
 
-        # Test update with parameter "recursive=True"
-        example_referable.update()
-        self.assertEqual(MockBackend.update_object.call_count, 2)
-        MockBackend.update_object.assert_has_calls([
-            mock.call(updated_object=example_referable,
-                      store_object=example_grandparent,
-                      relative_path=["exampleGrandparent", "exampleParent", "exampleReferable"]),
-            mock.call(updated_object=example_grandchild,
-                      store_object=example_grandchild,
-                      relative_path=[])
-        ])
-        MockBackend.update_object.reset_mock()
+        ref_child = ExampleReferable()
+        ref_child.id_short = "Child"
+        ref_child.parent = identifiable
 
-        # Test update with source != "" in example_referable
-        example_referable.source = "mockScheme:exampleReferable"
-        example_referable.update(recursive=False)
-        MockBackend.update_object.assert_called_once_with(
-            updated_object=example_referable,
-            store_object=example_referable,
-            relative_path=[]
-        )
-        MockBackend.update_object.reset_mock()
+        list1 = model.SubmodelElementList("List1", model.SubmodelElementList)
+        list2 = model.SubmodelElementList(None, model.Property, value_type_list_element=model.datatypes.Int)
+        prop1 = model.Property(None, model.datatypes.Int)
 
-        # Test update with no source available
-        example_grandparent.source = ""
-        example_referable.source = ""
-        example_referable.update(recursive=False)
-        MockBackend.update_object.assert_not_called()
+        list1.parent = ref_child
+        list1.add_referable(list2)
+        list2.add_referable(prop1)
 
-    def test_commit(self):
-        backends.register_backend("mockScheme", MockBackend)
-        example_referable = generate_example_referable_tree()
-        example_grandparent = example_referable.parent.parent
-        example_grandchild = example_referable.get_referable("exampleChild").get_referable("exampleGrandchild")
+        self.assertIs(ref_with_no_parent.get_identifiable_root(), None)
+        self.assertIs(identifiable.get_identifiable_root(), identifiable)
+        self.assertIs(ref_child.get_identifiable_root(), identifiable)
+        self.assertIs(list1.get_identifiable_root(), identifiable)
+        self.assertIs(list2.get_identifiable_root(), identifiable)
+        self.assertIs(prop1.get_identifiable_root(), identifiable)
 
-        # Test commit starting from example_referable
-        example_referable.commit()
-        self.assertEqual(MockBackend.commit_object.call_count, 2)
-        MockBackend.commit_object.assert_has_calls([
-            mock.call(committed_object=example_referable,
-                      store_object=example_grandparent,
-                      relative_path=["exampleParent", "exampleReferable"]),
-            mock.call(committed_object=example_grandchild,
-                      store_object=example_grandchild,
-                      relative_path=[])
-        ])
-        MockBackend.commit_object.reset_mock()
+    def test_get_id_short_path(self):
+        """
+        Tests the get_id_short_path() method of Referable objects.
 
-        # Test commit starting from example_grandchild
-        example_grandchild.commit()
-        self.assertEqual(MockBackend.commit_object.call_count, 2)
-        MockBackend.commit_object.assert_has_calls([
-            mock.call(committed_object=example_grandchild,
-                      store_object=example_grandparent,
-                      relative_path=["exampleParent", "exampleReferable", "exampleChild", "exampleGrandchild"]),
-            mock.call(committed_object=example_grandchild,
-                      store_object=example_grandchild,
-                      relative_path=[])
-        ])
-        MockBackend.commit_object.reset_mock()
+        Example structure:
+        - SMC: MySubmodelElementCollection
+            - Property: MySubProperty1
+            - Property: MySubProperty2
+            - SMC: MySubSubmodelElementCollection
+                - Property: MySubSubProperty1
+                - Property: MySubSubProperty2
+            - SML: MySubSubmodelElementList1
+                - Property: "MySubTestValue1"
+                - Property: "MySubTestValue2"
+            - SML: MySubSubmodelElementList2
+                - SML: MySubSubmodelElementList3
+                    - SMC: MySubmodelElementCollectionInSML3
+                        - Property: "MySubTestValue3"
+        """
+        MySubmodelElementCollection = model.SubmodelElementCollection("MySubmodelElementCollection")
+        MySubProperty1 = model.Property("MySubProperty1", model.datatypes.String)
+        MySubProperty2 = model.Property("MySubProperty2", model.datatypes.String)
+        MySubSubmodelElementCollection = model.SubmodelElementCollection("MySubSubmodelElementCollection")
+        MySubSubProperty1 = model.Property("MySubSubProperty1", model.datatypes.String)
+        MySubSubProperty2 = model.Property("MySubSubProperty2", model.datatypes.String)
+        MySubSubmodelElementList1 = model.SubmodelElementList("MySubSubmodelElementList1", model.Property,
+                                                              value_type_list_element=model.datatypes.String)
+        MySubTestValue1 = model.Property(None, model.datatypes.String)
+        MySubTestValue2 = model.Property(None, model.datatypes.String)
+        MySubSubmodelElementList2 = model.SubmodelElementList("MySubSubmodelElementList2", model.SubmodelElementList)
+        MySubSubmodelElementList3 = model.SubmodelElementList(None, model.SubmodelElementCollection)
+        MySubmodelElementCollectionInSML3 = model.SubmodelElementCollection(None)
+        MySubTestValue3 = model.Property("MySubTestValue3", model.datatypes.String)
 
-        # Test commit starting from example_grandchild after adding a source to example_referable
-        example_referable.source = "mockScheme:exampleReferable"
-        example_grandchild.commit()
-        self.assertEqual(MockBackend.commit_object.call_count, 3)
-        MockBackend.commit_object.assert_has_calls([
-            mock.call(committed_object=example_grandchild,
-                      store_object=example_referable,
-                      relative_path=["exampleChild", "exampleGrandchild"]),
-            mock.call(committed_object=example_grandchild,
-                      store_object=example_grandparent,
-                      relative_path=["exampleParent", "exampleReferable", "exampleChild", "exampleGrandchild"]),
-            mock.call(committed_object=example_grandchild,
-                      store_object=example_grandchild,
-                      relative_path=[])
-        ])
+        MySubmodelElementCollection.add_referable(MySubProperty1)
+        MySubmodelElementCollection.add_referable(MySubProperty2)
+        MySubmodelElementCollection.add_referable(MySubSubmodelElementCollection)
+        MySubSubmodelElementCollection.add_referable(MySubSubProperty1)
+        MySubSubmodelElementCollection.add_referable(MySubSubProperty2)
+        MySubmodelElementCollection.add_referable(MySubSubmodelElementList1)
+        MySubSubmodelElementList1.add_referable(MySubTestValue1)
+        MySubSubmodelElementList1.add_referable(MySubTestValue2)
+        MySubmodelElementCollection.add_referable(MySubSubmodelElementList2)
+        MySubSubmodelElementList2.add_referable(MySubSubmodelElementList3)
+        MySubSubmodelElementList3.add_referable(MySubmodelElementCollectionInSML3)
+        MySubmodelElementCollectionInSML3.add_referable(MySubTestValue3)
+
+        expected_id_short_paths = {
+            MySubmodelElementCollection: "MySubmodelElementCollection",
+            MySubProperty1: "MySubmodelElementCollection.MySubProperty1",
+            MySubProperty2: "MySubmodelElementCollection.MySubProperty2",
+            MySubSubmodelElementCollection: "MySubmodelElementCollection.MySubSubmodelElementCollection",
+            MySubSubProperty1: "MySubmodelElementCollection.MySubSubmodelElementCollection.MySubSubProperty1",
+            MySubSubProperty2: "MySubmodelElementCollection.MySubSubmodelElementCollection.MySubSubProperty2",
+            MySubSubmodelElementList1: "MySubmodelElementCollection.MySubSubmodelElementList1",
+            MySubTestValue1: "MySubmodelElementCollection.MySubSubmodelElementList1[0]",
+            MySubTestValue2: "MySubmodelElementCollection.MySubSubmodelElementList1[1]",
+            MySubSubmodelElementList2: "MySubmodelElementCollection.MySubSubmodelElementList2",
+            MySubSubmodelElementList3: "MySubmodelElementCollection.MySubSubmodelElementList2[0]",
+            MySubmodelElementCollectionInSML3: "MySubmodelElementCollection.MySubSubmodelElementList2[0][0]",
+            MySubTestValue3: "MySubmodelElementCollection.MySubSubmodelElementList2[0][0].MySubTestValue3",
+        }
+        for referable, expected_path in expected_id_short_paths.items():
+            self.assertEqual(referable.get_id_short_path(), expected_path)
 
     def test_update_from(self):
         example_submodel = example_aas.create_example_submodel()
@@ -270,20 +246,8 @@ class ReferableTest(unittest.TestCase):
         self.assertIs(example_submodel.namespace_element_sets[0], example_submodel.submodel_element)
         self.assertIs(example_relel.parent, example_submodel)
 
-        # Test source update
-        example_relel.source = "scheme:OldRelElSource"
-        other_submodel.source = "scheme:NewSource"
-        other_relel.source = "scheme:NewRelElSource"
-
-        example_submodel.update_from(other_submodel)
-        # Sources of the object itself should not be updated by default
-        self.assertEqual("", example_submodel.source)
-        # Sources of embedded objects should always be updated
-        self.assertEqual("scheme:NewRelElSource", example_relel.source)
-
     def test_update_commit_qualifier_extension_semantic_id(self):
         submodel = model.Submodel("https://acplt.org/Test_Submodel")
-        submodel.update()
         qualifier = model.Qualifier("test", model.datatypes.String)
         extension = model.Extension("test")
         collection = model.SubmodelElementCollection("test")
@@ -293,7 +257,6 @@ class ReferableTest(unittest.TestCase):
         submodel.add_qualifier(qualifier)
         submodel.add_extension(extension)
         submodel.add_referable(collection)
-        submodel.commit()
 
         self.assertEqual(next(iter(submodel.qualifier)), qualifier)
         self.assertEqual(next(iter(submodel.extension)), extension)
@@ -320,7 +283,6 @@ class ReferableTest(unittest.TestCase):
             next(iter(submodel.submodel_element))
         with self.assertRaises(StopIteration):
             next(iter(collection.value))
-        submodel.commit()
 
 
 class ExampleNamespaceReferable(model.UniqueIdShortNamespace, model.UniqueSemanticIdNamespace, model.Identifiable):
@@ -609,7 +571,7 @@ class ModelNamespaceTest(unittest.TestCase):
         with self.assertRaises(TypeError) as cm_3:
             self.namespace.get_referable(["List1", "0", "Prop1", "Test"])
         self.assertEqual("Cannot resolve id_short or index 'Test' at "
-                         f"Property[{self.namespace.id} / List1[0] / Prop1], "
+                         f"Property[{self.namespace.id} / List1[0].Prop1], "
                          "because it is not a UniqueIdShortNamespace!", str(cm_3.exception))
 
         self.namespace.get_referable(["List1", "0", "Prop1"])
@@ -696,7 +658,7 @@ class ModelNamespaceTest(unittest.TestCase):
         se_collection.add_referable(property)
         with self.assertRaises(model.AASConstraintViolation) as cm:
             property.id_short = None
-        self.assertEqual("id_short of Property[foo / property] cannot be unset, since it is already contained in "
+        self.assertEqual("id_short of Property[foo.property] cannot be unset, since it is already contained in "
                          "SubmodelElementCollection[foo] (Constraint AASd-117)", str(cm.exception))
         property.id_short = "bar"
 
@@ -947,7 +909,7 @@ class ModelReferenceTest(unittest.TestCase):
                                     model.Property)
         with self.assertRaises(TypeError) as cm_3:
             ref4.resolve(DummyObjectProvider())
-        self.assertEqual("Cannot resolve id_short or index 'prop' at Property[urn:x-test:submodel / list[0] / prop], "
+        self.assertEqual("Cannot resolve id_short or index 'prop' at Property[urn:x-test:submodel / list[0].prop], "
                          "because it is not a UniqueIdShortNamespace!", str(cm_3.exception))
 
         with self.assertRaises(AttributeError) as cm_4:
@@ -1024,13 +986,13 @@ class ModelReferenceTest(unittest.TestCase):
         submodel.submodel_element.remove(collection)
         with self.assertRaises(ValueError) as cm:
             ref3 = model.ModelReference.from_referable(prop)
-        self.assertEqual("The given Referable object is not embedded within an Identifiable object", str(cm.exception))
+        self.assertEqual("The given Referable object is not embedded within an Identifiable object",
+                         str(cm.exception).split(":")[0])
 
-        # Test creating a reference to a custom Referable class
-        class DummyThing(model.Referable):
+        # Test creating a reference to a custom SubmodelElement class
+        class DummyThing(model.SubmodelElement):
             def __init__(self, id_short: model.NameType):
-                super().__init__()
-                self.id_short = id_short
+                super().__init__(id_short)
 
         class DummyIdentifyableNamespace(model.Submodel, model.UniqueIdShortNamespace):
             def __init__(self, id_: model.Identifier):
@@ -1041,7 +1003,7 @@ class ModelReferenceTest(unittest.TestCase):
         identifable_thing = DummyIdentifyableNamespace("urn:x-test:thing")
         identifable_thing.things.add(thing)
         ref4 = model.ModelReference.from_referable(thing)
-        self.assertIs(ref4.type, model.Referable)
+        self.assertIs(ref4.type, model.SubmodelElement)
 
 
 class AdministrativeInformationTest(unittest.TestCase):
